/* 
modified from original code by Klaus, DJ7OO  
version for LCD shield by ElecFreaks    
using internal Rotary Encoder (#2, #3) and Joystick (#A0) 

Todo:   switchable IF offset 
        sidebandselection ( incl. display )
        using joystick pusbutton     
              
JOY:    #A0  in   Analog Voltage Input from Joystick    
              switch "left"   = Ham Band-
              switch "right"  = Ham Band+
              switch "up"     = Frq Step+
              switch "down"   = Frq Step-   
              switch "Push"   = not yet used
                  
DDS:          #A1  spare 
              #A2  WCLK
              #A3  FQUD
              #A4  DATA 
              #A5  RESET
*/

#include <Rotary.h>
#include <LiquidCrystal.h>  

#define btnNone 5
#define pulseHigh(pin) {digitalWrite(pin, HIGH); digitalWrite(pin, LOW); }

//set DDS pins
#define W_CLK A2  // AD9850 load clock pin (FCLK)
#define FQ_UD A3   // freq update pin (FQUD)
#define DATA  A4   // serial data load pin (DATA)
#define RESET A5   // reset pin (RST) 

Rotary r = Rotary(2,3); // sets the pins the rotary encoder uses.  
LiquidCrystal lcd(8, 9, 4, 5, 6, 7);    // settings for EleyFreaks Key Shield

int_fast32_t rx = 14100000; // Starting frequency of VFO 14.100 MHz
int_fast32_t rx_old;        // variable to hold the updated frequency
int_fast32_t rx_ofs;        // command frequency for DDS incl. possible IF offset 
int_fast32_t step1 = 100;   // starting VFO update increment in HZ.

float rx_disp;
int  band = 4;                          // starting in 20m Band
int  band_old;
int  stp = 2;                           // starting with 100Hz steps 
int  stp_old;
int mode = 1;                           // not yet implemented  
int mode_old;
int adc_key_in = 0;

////////////////////////////////////////////////////////////
void setup() {    
  Serial.begin(9600);    
  Serial.println("DDS-Generator V1.0");
  pinMode(A0,INPUT); 

  // set Backlight ON 
  pinMode(10,OUTPUT);
  digitalWrite(10,1);  

  lcd.begin (16,2);  // display init
  lcd.setCursor(0,0); // Char 1, Line 1
  lcd.print(" DDS-GENERATOR  ");
  lcd.setCursor(0,1); // Char 1, Line 2
  lcd.print(" (C)DJ7OO 2014  ");
  delay(3000);
  lcd.clear();

  PCICR |= (1 << PCIE2);
  PCMSK2 |= (1 << PCINT18) | (1 << PCINT19);
  sei();

  pinMode(FQ_UD, OUTPUT);
  pinMode(W_CLK, OUTPUT);
  pinMode(DATA, OUTPUT);
  pinMode(RESET, OUTPUT); 

  pulseHigh(RESET);
  pulseHigh(W_CLK);
  pulseHigh(FQ_UD);  // this pulse enables serial mode on the AD9850 - Datasheet page 12.
 }

////////////////////////////////////////////////////////////  
void loop() {
  read_Joystick();
  read_Band_Select();
  read_Step_Select();

  if (rx != rx_old)  {    
  Serial.print("RX  frequency : "); 
  Serial.print(rx);
  Serial.println("Hz"); 

  lcd.setCursor(0,0); // Char 1, Line 1
  lcd.print("Frq.:");
  rx_disp = rx;
  rx_disp = rx_disp/1000;
  lcd.print(rx_disp,2); 
  lcd.print("KHz ");
  rx_old = rx;
  rx_ofs = rx + 0;             // add IF offset frequency if required    
  Serial.print("DDS frequency : "); 
  Serial.print(rx_ofs);
  Serial.println("Hz"); 
  sendFrequency(rx_ofs);       // frequency for controlling the DDS
  }  
}

////////////////////////////////////////////////////////////
// frequency calc 
void sendFrequency(double frequency) {  
  int32_t freq = frequency * 4294967295/125000000;  // note 125 MHz clock on AD9850.  You can make 'slight' tuning variations here by adjusting the clock frequency.
  for (int b=0; b<4; b++, freq>>=8) {
    tfr_byte(freq & 0xFF);
  }
  tfr_byte(0x000);   // Final control byte, all 0 for AD9850 chip
  pulseHigh(FQ_UD);  // Done!  Should see output
}

////////////////////////////////////////////////////////////
// transfers a byte, a bit at a time, LSB first to the AD9850 via serial DATA line
void tfr_byte(byte data) {
  for (int i=0; i<8; i++, data>>=1) 
  {
    digitalWrite(DATA, data & 0x01);
    pulseHigh(W_CLK);   //after each bit sent, CLK is pulsed high
  }
}

/////////////////////////////////////////////////////////////////
void read_Step_Select() {
  if(stp==7)  
  {stp=1; }
  if(stp==0)  
  {stp=6; }
  
  if(stp!=stp_old)  
     {
  if (stp==1) 
     {   
     step1 = 10;
 
  Serial.println("selected step : 10Hz");
  lcd.setCursor(0,1); // Char 1, Line 2
  lcd.print("Step:10Hz      ");
     }
 else if (stp==2) 
     {   
     step1 = 100;
  Serial.println("selected step : 100Hz");
  lcd.setCursor(0,1); // Char 1, Line 2
  lcd.print("Step:100Hz     ");
     }
 else if (stp==3) 
     {   
     step1 = 1000;
  Serial.println("selected step : 1KHz");
  lcd.setCursor(0,1); // Char 1, Line 2
  lcd.print("Step:1KHz      ");
     }
 else if (stp==4) 
     {   
     step1 = 10000;
  Serial.println("selected step : 10KHz");
  lcd.setCursor(0,1); // Char 1, Line 2
  lcd.print("Step:10KHz     ");
     }
  else if (stp==5) 
     {   
     step1 = 100000;
  Serial.println("selected step : 100KHz");
  lcd.setCursor(0,1); // Char 1, Line 2
  lcd.print("Step:100KHz    ");
     }
  else if (stp==6) 
     {   
     step1 = 1000000;
  Serial.println("selected step : 1MHz");
  lcd.setCursor(0,1); // Char 1, Line 2
  lcd.print("Step:1MHz      ");
       }
     delay(500); 
     stp_old = stp;  
  }
}
  
/////////////////////////////////////////////////////////////////
void read_Band_Select(){
  if(band==7)  
    {band=1; }
  if(band==0)  
    {band=6; }
  if(band!=band_old)  
    { 
      if (band==1) 
    { rx = 1800000L; Serial.println("selected band : 160m"); }
 else if (band==2) 
    { rx = 3600000L; Serial.println("selected band : 80m"); }
 else if (band==3) 
    { rx = 7100000L; Serial.println("selected band : 40m"); }
 else if (band==4) 
   { rx = 14100000L; Serial.println("selected band : 20m"); }
 else if (band==5) 
   { rx = 21100000L; Serial.println("selected band : 15m"); }
 else if (band==6) 
   { rx = 28500000L; Serial.println("selected band : 10m"); } 
     delay(500); 
     band_old = band;  
  }
}
  
////////////////////////////////////////////////////////////////
// read the joystick
int read_Joystick()
{
adc_key_in = analogRead(0); // read the value from the sensor 
// Serial.println(adc_key_in);

if (adc_key_in > 1000) return btnNone; 
if (adc_key_in < 50) return band=band-1; 
if (adc_key_in < 150) return stp=stp+1; 
if (adc_key_in < 250) return band=band+1; 
if (adc_key_in < 450) return mode=mode+1;   // not yet implemented
if (adc_key_in < 700) return stp=stp-1; 
return btnNone; // when all others fail, return this...
}

////////////////////////////////////////////////////////////////////////////////
ISR(PCINT2_vect) {
  unsigned char result = r.process();
  if (result) {    
    if (result == DIR_CW){rx=rx+step1;}
    else {rx=rx-step1;};       
      if (rx >=40000000){rx=rx_old;}; // UPPER VFO LIMIT 40MHz
      if  (rx <= 100000){rx=rx_old;}; // LOWER VFO LIMIT 100KHz
  }
}

